(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "react"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.lipidReactHookGenerator = void 0;
    const react_1 = require("react");
    const useIsMounted = () => {
        const isMounted = (0, react_1.useRef)(false);
        (0, react_1.useEffect)(() => {
            isMounted.current = true;
            return () => {
                isMounted.current = false;
            };
        }, []);
        return isMounted;
    };
    function lipidReactHookGenerator(lipidState) {
        return (properties) => {
            const isMounted = useIsMounted();
            const [state, setState] = (0, react_1.useState)(Object.fromEntries(properties.reduce((acc, key) => {
                acc.push([key, lipidState.get(key)]);
                return acc;
            }, [])));
            (0, react_1.useEffect)(() => {
                const obs = lipidState
                    .on(properties)
                    .subscribe(({ state }) => {
                    if (isMounted.current) {
                        setState(state);
                    }
                });
                return () => {
                    obs.unsubscribe();
                };
            }, []);
            return state;
        };
    }
    exports.lipidReactHookGenerator = lipidReactHookGenerator;
});
