var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "rxjs", "rxjs/operators", "lodash/intersection", "lodash/isEqual", "lodash-es/cloneDeep"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Lipid = void 0;
    const rxjs_1 = require("rxjs");
    const operators_1 = require("rxjs/operators");
    const intersection_1 = __importDefault(require("lodash/intersection"));
    const isEqual_1 = __importDefault(require("lodash/isEqual"));
    const cloneDeep_1 = __importDefault(require("lodash-es/cloneDeep"));
    class Lipid {
        constructor(state) {
            if (state && typeof state !== 'object') {
                throw Error('new Lipid() "state" argument must be an object.');
            }
            this.__defaultState = state;
            this.__obs = new rxjs_1.Subject();
            this.__state = state || {};
            this.onSetBefore = state => state;
            this.onSetAfter = () => { };
        }
        on(props = []) {
            return this.__obs.pipe((0, operators_1.filter)(({ state, prevState, delta }) => {
                if (!props || !props.length)
                    return true;
                const keyProps = (0, intersection_1.default)(props, Object.keys(delta));
                return !!keyProps.reduce((final, prop) => {
                    if (!(0, isEqual_1.default)(state[prop], prevState[prop])) {
                        return [...final, prop];
                    }
                    return final;
                }, []).length;
            }), (0, operators_1.map)(({ state, prevState, delta }) => {
                return ({ state, prevState });
            }));
        }
        get(prop) {
            return prop ? this.__state[prop] : this.__state;
        }
        set(newState, options = { emit: true, clear: false }) {
            const { clear, emit } = options;
            let delta = typeof newState === 'function'
                ? newState(this.__state)
                : newState;
            if (!(typeof delta === 'object' && delta !== null)) {
                throw new TypeError(`[${this.constructor.name}] "state" argument must be an object`);
            }
            delta = this.onSetBefore((0, cloneDeep_1.default)(this.__state), delta);
            const prevState = Object.assign({}, this.__state);
            this.__state = clear
                ? Object.assign({}, delta)
                : Object.assign({}, this.__state, delta);
            this.onSetAfter(this.__state, delta);
            if (emit) {
                this.__obs.next({ state: this.__state, prevState, delta });
            }
            return this.__state;
        }
        revertToDefault(options = { emit: true, clear: true }) {
            return this.set(this.__defaultState, options);
        }
        setDefault(state, options = { emit: true, clear: true }) {
            this.__defaultState = state;
            return this.set(this.__defaultState, options);
        }
    }
    exports.Lipid = Lipid;
});
